package be.iminds.ilabt.jfed.experimenter_gui.dialogs;

import be.iminds.ilabt.jfed.experiment.ExperimentController;
import be.iminds.ilabt.jfed.experimenter_gui.events.OpenESpecEvent;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.UIUtils;
import be.iminds.ilabt.jfed.gui_preferences.HLPreferenceKey;
import be.iminds.ilabt.jfed.lowlevel.authority.finder.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUserProvider;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.GuiceFXMLUtil;
import com.google.common.eventbus.EventBus;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.*;
import javafx.scene.paint.Color;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Inject;
import java.net.MalformedURLException;


public class RunSshCommandDialog extends Dialog<ButtonType> {
    private static final Logger LOG = LoggerFactory.getLogger(RunSshCommandDialog.class);

    @Nonnull private final RunSshCommandController controller;
    @Nonnull private final JFedPreferences jFedPreferences;
    @Nonnull private final AuthorityFinder authorityFinder;
    @Nonnull private final GeniUserProvider geniUserProvider;
    @Nonnull private final EventBus eventBus;

    //The experimentController of the running experiment
    private ExperimentController experimentController;

    @Inject
    public RunSshCommandDialog(@Nonnull GuiceFXMLUtil guiceFXMLUtil,
                               @Nonnull JFedPreferences jFedPreferences,
                               @Nonnull AuthorityFinder authorityFinder,
                               @Nonnull GeniUserProvider geniUserProvider,
                               @Nonnull EventBus eventBus) {
        this.jFedPreferences = jFedPreferences;
        this.authorityFinder = authorityFinder;
        this.geniUserProvider = geniUserProvider;
        this.eventBus = eventBus;

        GuiceFXMLUtil.Result<?, RunSshCommandController> result =
                guiceFXMLUtil.createFromFXML(RunSshCommandController.class);

        this.controller = result.getController();

        setTitle("Multi Command");
        setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.TERMINAL, Color.DARKGREEN));
        setHeaderText("Execute a command on multiple nodes using SSH");
        //TODO: is manual decoration pane wrapping still needed?
        getDialogPane().setContent(UIUtils.wrapWithDecorationPane(result.getRoot()));
        setResizable(true);

        ButtonType runCommandAction =
                new ButtonType("Run SSH Command", ButtonBar.ButtonData.OK_DONE); //this::checkAndOpenESpec);
        getDialogPane().getButtonTypes().setAll(runCommandAction, ButtonType.CANCEL);

        Button runCommandButton = (Button) getDialogPane().lookupButton(runCommandAction);
        runCommandButton.disableProperty().bind(controller.validProperty().not());

        EventHandler<ActionEvent> openESpecEventHandler = event -> {
                    if (controller.isValid())
                        runSshCommand();
                    event.consume();
                };
        runCommandButton.addEventFilter(ActionEvent.ACTION, openESpecEventHandler);
        controller.setOnAction(openESpecEventHandler);
    }



    /**
     * This method uses the inputs from the dialog to start the experiment.
     * These inputs should be validated before calling this function!
     */
    private void runSshCommand() {
        hide();

        try {
            OpenESpecEvent openESpecEvent = new OpenESpecEvent(experimentController, controller.getESpecBundle());
            LOG.info("User pressed \"Run Ssh command\" button: {}", openESpecEvent);
            eventBus.post(openESpecEvent);
//            jFedPreferences.setString(HLPreferenceKey.PREF_LAST_OPENED_ESPEC, controller.getESpecSourceString());
        } catch (MalformedURLException e) {
            LOG.error("User pressed \"Run Ssh command\" button, but caused MalformedURLException.", e);
        }
    }

    public void setExperimentController(@Nonnull ExperimentController experimentController) {
        this.experimentController = experimentController;
        this.controller.setExperimentController(experimentController);
    }
}
